#ifndef ARRAY_CPP
#define ARRAY_CPP

/**
* @file   Array.cpp
* @class  Array
* @author Morten Hjorth-Jensen (morten.hjorth-jensen@fys.uio.no) and Islen Vallejo
(islenv@fys.uio.no)
* @date   May, 2010
* @brief  Implementation of class used for manipulating up to two-dimensional arrays.
**/


#include "Array.h"

// Copy constructor
template <class T>
Array<T>::Array(const Array<T>& array){
	*this = array;	
}


template <class T>
bool Array<T>::indexOk(int i, int j) const{
	if((i < 0 || i>size[0]-1)|| (j< 0 || j >size[1]-1)){
		std::cerr << "Index out of bounds" << std::endl;
		return false;
	}else 
		return true;
}



// Change the dimensions of an array
template <class T>
bool Array<T>::redim(int ni, int nj, int nk, int nl, int nm, int nn){

	if(data != NULL){
		length = 0;
		ndim   = 0;
		delete[] data;
	}
	
	allocate(ni,nj,nk,nl,nm,nn);
	
	return true;
	
}



/********************************************************/
/* 				       USER-DEFINED OPERATORS 								*/
/********************************************************/
// Assigment operator
template <class T>
Array<T>& Array<T>::operator=(const Array<T>& array){
			
		if(this != &array){									// beware of self-assignment v=v
			ndim   	= array.ndim;
			length 	= array.length;
			data 		= new T[array.length];
			
			for(int i=0; i<array.length; i++)
				data[i] = array.data[i];
		}
					
		for(int i=0; i<ndim; i++){
			size[i] = array.size[i];
		}
		
		return *this;
	}
	

// Addition operator
template <class T>
Array<T> Array<T>::operator+(const Array<T>& array){

	Array<T> sum = *this;
  
  for(int i=0; i<length; i++){
    sum.data[i] = data[i] + array.data[i];
  }
  
  return sum;
}
    

// Substraction operator
template <class T>
Array<T> Array<T>::operator-(const Array<T>& array)const{
   
	Array<T> rest = *this; // The same as: Array<T> rest(nrows, ncols);
	rest -= array;

	return rest;
}


// Assigment by addition operator
template <class T>
Array<T>& Array<T>::operator+=(const Array<T>& w){  // v += w
	if(length != w.length) std::cout << "Bad vector sizes" << std::endl;
	
	for(int i=0; i<length; i++)
	data[i] += w.data[i]; // This possible because we have overloaded the operator[]
	return *this;
} // add a vector to the current one


// Assignment by substraction operator
template <class T>
Array<T>& Array<T>::operator-=(const Array<T>& w){  // v -= w
	if(length != w.length) std::cout << "Bad vector sizes" << std::endl;
	
	for(int i=0; i<length; i++){
		data[i] -= w.data[i];// This possible because we have overloaded the operator[]
	}
	return *this;
}


// Assignment by scalar product operator
template <class T>
Array<T>& Array<T>::operator*=(double scalar){    // v *= a
	for(int i=0; i<length; i++)
		data[i] *= scalar;
	return *this;
}


template <class T>
Array<T>& Array<T>::operator/=(double scalar){    // v /= a
	for(int i=0; i<length; i++)
		data[i] /= scalar;
	return *this;
}

#endif
